; ----------------------------------------------------------------------------
;
; Name, Version & History
; Examiner
; Author	TG Tan		www.angelfire.com\space2\tgtan
; V1.0		11/2/07		1st version
; V1.01		16/2/07		Modify so that the visulisation thresholds stick when the user changes them
; V1.1		17/3/07		Modify so that the date of the observation is written to the Results filename
; V1.11		20/3/07		Bug fix - previously the target name recorded in the results file was the next target rather than the current one
; V1.12		10/9/07		Put a delay between opening IRIS windows to avoid timing conflicts
; V2.0		12/1/08		Modify so reference images are in another, fixed directory
; V2.01		7/6/08		Change position of ref window to be closer to obj window, made windows slightly smaller, use the first targetpoint file found without asking
; V2.02		5/8/08		Changed drive letter of ref images folder to G:
; 
; Author:         TG Tan
;
; Script Function:
;  Works with 2 IRIS windows to help with examination of galaxy images vs reference DSS images
;  Writes a result file with date of observation, list of galaxies, and whether positive (+), negative (-), or missed the target (?)
;
; Assumptions:
;
; 1.  Uses the TargetPoint data file to identify successive image files
; 2.  DSS images have the format "<galaxy name>_o.fit", subject images are "<galaxy name>xn.fit" where n= 1, 2 etc.
; ----------------------------------------------------------------------------
Global $ver = "2.02"
;**************************************************************
;  some initial preparations...
;***************************************************************

#include <Date.au3>
#include <file.au3>
#include <GuiConstants.au3>

HotKeySet("^!x", "My_Final_Exit") ; exit the script if Ctrl Alt x is pressed
HotKeySet("^t", "Next_Target") ; Ctrl T to go to previous image
HotKeySet("^{NUMPADADD}", "Next_Image") ; Ctrl + to go to next image
HotKeySet("^{NUMPADSUB}", "Prev_Image") ; Ctrl - to go to previous image

; declare variables


Global $path = 	"G:\AstroData\SN_Search"		; location of the input and output files
Global $ref_image_path = "G:\AstroData\SN_Search\Ref_Images"		; location of reference images

Global $cur_target		; string with the name of the current target - no spaces allowed
Global $image_no = 0		; the series number of the current object image file 
Global $targetpoint_file
Global $result_file
Global $targets_total		; the number of targets identified
Global $targets_done
Global $date_string			; this is the date of observation, taken from the name of the TargetPoint file
Global $log_on = $GUI_UNCHECKED		; if checked, write actions to log file
Global $initial_run = 1			; indicates that this is the initial run of the image loading routine, so need to set the visu levels

Global $iris_console = 1006
Global $iris_console_button_x = 281	; coord of the console button when the main IRIS window is at the 0,0 posn
Global $iris_console_button_y = 84
Global $iris_File_button_x = 16
Global $iris_File_button_y = 41
Global $iris_default_dir_ctrl = 1113	; the control for the IRIS default directory in the Settings box
Global $ref_pane_x = 333			; where to click to make the reference pane active
Global $ref_pane_y = 13
Global $obj_pane_x = 914			; where to click to make the image pane active
Global $obj_pane_y = 13


;*****************************************************************************
; initialise some variables
;*****************************************************************************

$targets_total = 0		; a temporary number


;*****************************************************************************
; This block contains code for the script graphical interface
;******************************************************************************


opt("GuiOnEventMode", 1) ; change to On Event mode
$main_window = GUICreate("Examiner, V" & $ver & " by TG Tan", 360, 380, 360, 590)

; input fields

GUICtrlCreateLabel("Input and output file path:", 10, 21)
$path_box = GUICtrlCreateInput($path, 170, 16, 160, 20, $ES_AUTOHSCROLL)

; make a Browse button for the path

$browse_button = GUICtrlCreateButton("Browse", 270, 41, 50, 30)
GUICtrlSetOnEvent($browse_button, "Browse_path") ; when button is pressed, locate the desired path

$log_checkbox = GUICtrlCreateCheckbox(" Log this run?", 30, 51)

;*********************************************************************************
; the status display part of the GUI
;*********************************************************************************

; buttons

$ok_button = GUICtrlCreateButton("RUN", 70, 80, 70, 40)
GUICtrlSetOnEvent($ok_button, "Execute_Main") ; when button is pressed, proceed
$stop_button = GUICtrlCreateButton("CANCEL", 170, 80, 70, 40)
GUICtrlSetOnEvent($stop_button, "My_Final_Exit") ; ends the script



$next_target_button = GUICtrlCreateButton("Next Target (Ctrl T)", 70, 140, 100, 50)
GUICtrlSetOnEvent($next_target_button, "Next_Target") ; when button is pressed, goto next target
$prev_image_button = GUICtrlCreateButton("Prev (Ctrl -)", 200, 140, 70, 30)
GUICtrlSetOnEvent($prev_image_button, "Prev_Image")
$next_image_button = GUICtrlCreateButton("Next (Ctrl +)", 280, 140, 70, 30)
GUICtrlSetOnEvent($next_image_button, "Next_Image") ; 

; tracks the current Target, and the current image in the series being displayed

$cur_target_label = GUICtrlCreateLabel("Galaxy", 10, 200, 160, 50)
GUICtrlSetFont($cur_target_label, 12, 600)
$cur_target_box = GUICtrlCreateInput($cur_target, 80, 198, 90, 25, $ES_READONLY)
GUICtrlSetFont($cur_target_box, 10, 800)

$image_no_label = GUICtrlCreateLabel("Image no.", 180, 200, 160, 50)
GUICtrlSetFont($image_no_label, 12, 600)
$image_no_box = GUICtrlCreateLabel($image_no, 270, 195, 160, 50, $ES_READONLY)
GUICtrlSetFont($image_no_box, 18, 800)


GUICtrlCreateGroup("Supernova present?", 5, 230, 350, 50)		; start group

$radio_yes = GUICtrlCreateRadio("Yes", 50, 250)
$radio_no = GUICtrlCreateRadio("No", 100, 250)
$radio_undef = GUICtrlCreateRadio("Undefined", 150, 250)

GUICtrlCreateGroup("" -99, -99, 1, 1)		; close group

GUICtrlSetState($radio_undef, $GUI_CHECKED)		; to start with, all detections are undefined

; make a display that keeps track of the no. of targets done

$targets_done_label = GUICtrlCreateLabel("Targets done:", 20, 312, 160, 50)
GUICtrlSetFont($targets_done_label, 12, 600)
$targets_done_box = GUICtrlCreateInput($targets_done, 150, 305, 70, 32, $ES_READONLY)
GUICtrlSetFont($targets_done_box, 20, 800)

$targets_total_label = GUICtrlCreateLabel("of total", 230, 312, 160, 50)
GUICtrlSetFont($targets_total_label, 12, 600)
$targets_total_number = GUICtrlCreateLabel($targets_total, 300, 307, 160, 50)
GUICtrlSetFont($targets_total_number, 20, 800)


$exit_label = GUICtrlCreateLabel("Once RUN activated, Ctrl-Alt-X to Exit.", 10, 350, 380, 110)
GUICtrlSetFont($exit_label, 9, 800, 2)

; housekeeping functions for the main window

GUISetOnEvent($GUI_EVENT_CLOSE, "My_Final_Exit") ; pressing X ends the script
GUISetOnEvent($GUI_EVENT_MINIMIZE, "Minimize")
GUISetOnEvent($GUI_EVENT_RESTORE, "Restore")

GUISetState(@SW_SHOW)
Wait_For_Input()

;-------------------------------------------------------------------------------------------------------------
;
;  End of main program block
;
;-------------------------------------------------------------------------------------------------------------

;******************************************************************
; this function is a loop that keeps the script going while data is being input
;******************************************************************

Func Wait_For_Input()
	
	While 1
		Sleep(500)		; this delay tries to avoid excessive processor churning
	WEnd
EndFunc   ;==>Wait_For_Input


;************************************************************************************
;
;  this contains the main loop of the program
;
;***********************************************************************************

Func Execute_Main()
	
	; the Run button stops working here
	
	;GUICtrlSetState($stop_button, $GUI_DISABLE)
	GUICtrlSetState($ok_button, $GUI_DISABLE)
	
	; assign parameters
	
	$path = GUICtrlRead($path_box)
	$log_on = GUICtrlRead($log_checkbox)
	$targets_done = 0
	
	If StringRight($path, 1) <> "\" Then $path = $path & "\"		; make sure the path parameter has the correct form 
	

	
	;*************************************************************************
	; Open the target list for reading, if it's not there, exit
	;*************************************************************************
	$file_search = FileFindFirstFile($path & "targetpoint*.txt")
	
	; Check if the search was successful
	If $file_search = -1 Then
		MsgBox(0, "Error", "The target list targetpoint*.txt not found")
		Exit
	EndIf

	;While 1
		
		$targetpoint_filename = FileFindNextFile($file_search) 
		$date_string = StringMid($targetpoint_filename, 12, 8)
		;If MsgBox(4, "File Found", "OK to use " & $targetpoint_filename & " ?") = 6 Then ExitLoop
	;WEnd
		
	; Close the search handle
	FileClose($file_search)
	
	$targets_total = _FileCountLines($path & $targetpoint_filename)
	$targetpoint_file = FileOpen($path & $targetpoint_filename, 0)
	GUICtrlSetData($targets_total_number, $targets_total)


	;************************************************************
	; open files for writing
	;************************************************************
		
	If $log_on = $GUI_CHECKED Then
		$result_filename = "results" & $date_string & ".txt"
		$result_file = FileOpen($path & $result_filename, 2)	
		

	EndIf
	
	;******************************************************************
	; Open IRIS windows
	;******************************************************************
	
	Open_IRIS(170,0, 372, 484, 0, 493, 0, 613, $ref_image_path)		;x,y main window, sizex, sizey main window, x, y console, x,y threshold sliders
	Sleep(1000)
	Open_IRIS(552, 0, 713, 630, 752, 646, 752, 767, $path)	; this is the image window, the one before the DSS ref window
	
	;*****************************************************************
	; Read target ID, display its DSS image, and the first object image
	;*****************************************************************
	
	Read_and_Display()		; read and display the DSS image for the current target as well as the first object image
	$initial_run = 0		; subsequent image loads do not change the visu levels

	; the buttons start working again
	
	GUICtrlSetState($stop_button, $GUI_ENABLE)
	GUICtrlSetState($ok_button, $GUI_ENABLE)
	
	Return								; go back to within the original calling loop 
EndFunc   ;==>Execute_Main

;************************************************************************
; Function to read the input file at the current line, id the target, display the DSS ref image, and the first object image
;************************************************************************
Func Read_and_Display()
	Local $input_line[7] ; this receives the parsed input line from the catalog file
	Local $cur_target_line
	$cur_target_line = FileReadLine($targetpoint_file)
	$input_line = StringSplit($cur_target_line, " ")
	
	$cur_target = $input_line[1]
	GUICtrlSetData($cur_target_box, $cur_target)
	
	IRIS_Load("ref")
	If $initial_run Then ControlSend("Command", "", $iris_console, "visu 0 25000 "& "{ENTER}")		;sets the visu levels for the DSS pane
	IRIS_Load("obj")
	If $initial_run Then ControlSend("Command", "", $iris_console, "visu 1000 3000 "& "{ENTER}")

EndFunc		;==>Read_and_Display

;******************************************************************************
; Function to open an IRIS window, enter the current directory then size and position them as specified
;******************************************************************************
Func Open_IRIS($winx, $winy, $sizex, $sizey, $consolex, $consoley, $thresholdx, $thresholdy, $file_path)
	Run("C:\Program Files\iris\iris.exe")
	WinActivate("Iris")
	Sleep(500)
	WinWaitActive("Iris")			; wait till it is open
	WinMove("Iris", "", 0, 0, 380, 490)		; temporary position and size
	Sleep(500)
		
	; specify the current directory
	
	Send("^r")				; Cntrl R opens the settings dialog
	ControlSetText("Settings", "", $iris_default_dir_ctrl, $file_path)
	Send("{ENTER}")
	Sleep(500)
	
	; open the subsidiary windows
	MouseClick("left", $iris_console_button_x, $iris_console_button_y)
	Sleep(500)
	WinMove("Command", "", $consolex, $consoley)
	WinMove("Iris", "", $winx, $winy, $sizex, $sizey)
	WinMove("Threshold","", $thresholdx, $thresholdy)
		
EndFunc		;==>Open_IRIS

;***************************************************************************************
; Function to load an image into IRIS.  Loads the reference pane is $pane = "ref" and the object pane if $pane = "obj"
;***************************************************************************************
Func IRIS_Load($pane)
	
	Select
		Case $pane = "ref"
			MouseClick("left", $ref_pane_x, $ref_pane_y)
			WinWaitActive("Iris")
			If $initial_run Then ControlSend("Command", "", $iris_console, "{ENTER}")		; this extra ENTER is needed to make sure the command prompt is present
			ControlSend("Command", "", $iris_console, "load " & $cur_target & "_o" & "{ENTER}")
			;ControlSend("Command", "", $iris_console, "visu 0 25000 "& "{ENTER}")
		Case $pane = "obj"
			$image_no = $image_no + 1
			Image_Load()
			;ControlSend("Command", "", $iris_console, "visu 1000 3000 "& "{ENTER}")
	EndSelect
	
			
	
EndFunc		;==>IRIS_Load

;***************************************************************************************
; Function to load an object image into IRIS object pane.  
;***************************************************************************************
Func Image_Load()
	
	MouseClick("left", $obj_pane_x, $obj_pane_y)
	WinWaitActive("Iris")
	If $initial_run Then ControlSend("Command", "", $iris_console, "{ENTER}")		; this extra ENTER is needed to make sure the command prompt is present			
	ControlSend("Command", "", $iris_console, "load " & $cur_target & "x" & $image_no & "{ENTER}")
	GUICtrlSetData($image_no_box, $image_no)
	
EndFunc		;==>Image_Load

;***************************************************************************************
; Function to read the next target, display it and the first object image
;***************************************************************************************
Func Next_Target()
	Local $detection		; records if a supernova is detected
	Local $prev_target		; the target for which the detection was recorded
	$prev_target = $cur_target
	$image_no = 0
	Read_and_Display()
	$targets_done = $targets_done + 1
	GUICtrlSetData($targets_done_box, $targets_done)
	
	; records the result of the check for a supernova when the Next Target button is pressed
	
	If $log_on = $GUI_CHECKED Then
		Select
			Case GUICtrlRead($radio_yes) = $GUI_CHECKED
				$detection = "yes"
			Case GUICtrlRead($radio_no) = $GUI_CHECKED
				$detection = "no"
			Case GUICtrlRead($radio_undef) = $GUI_CHECKED
				$detection = "undef"
		EndSelect
		Write_Result_File($prev_target, $detection)
	EndIf
	
	
EndFunc		;==>Next_target

;***************************************************************************************
; Function to display the next in the series of object images
;***************************************************************************************
Func Next_Image()
	$image_no = $image_no + 1
	Image_Load()
	
EndFunc		;==>Next_Image


;***************************************************************************************
; Function to display the previous in the series of object images
;***************************************************************************************
Func Prev_Image()
	$image_no = $image_no - 1
	Image_Load()
	
EndFunc		;==>Prev_Image

;*****************************************************************
; Write the selected target details to the appropriate files
;*****************************************************************

Func Write_Result_File($target, $SN_detect)

	FileWriteLine($result_file, $target & " " & $SN_detect)
EndFunc		;==>Write_Result_File


;***********************************************
; window housekeeping functions
;***********************************************

Func Minimize()
	GUISetState(@SW_MINIMIZE)
EndFunc   ;==>Minimize

Func Restore()
	GUISetState(@SW_RESTORE)
EndFunc   ;==>Restore

Func Close_Window()
	GUIDelete()
EndFunc   ;==>Close_Window


;****************************************************************************
; Browse for the file save path and update the path input field when done
;***************************************************************************

Func Browse_path()
	$newpath = FileSelectFolder("Select save path", "", 1, $path)
	If $newpath <> "" Then $path = $newpath
	GUICtrlSetData($path_box, $path)
EndFunc


;***********************************************
; exit
;************************************************
 
Func My_Final_Exit()
	; complete the log file
	If $log_on = $GUI_CHECKED Then
		FileClose ($result_file)
	EndIf
	
	FileClose ($targetpoint_file)
	Exit; this completely terminates the script and closes the interface window
EndFunc   ;==>My_Final_Exit

;**************************************************************************************************
;  END of the script.
;**************************************************************************************************